@@ -139,9 +139,11 @@ class WorkerStatus {
 
   void WaitOnPauseBarrier() {
 #ifdef HAVE_PTHREAD_BARRIERS
+    pthread_rwlock_rdlock(&pause_rwlock_);
     int error = pthread_barrier_wait(&pause_barrier_);
     if (error != PTHREAD_BARRIER_SERIAL_THREAD)
       sat_assert(error == 0);
+    pthread_rwlock_unlock(&pause_rwlock_);
 #endif
   }
 
@@ -188,8 +190,8 @@ class WorkerStatus {
   Status status_;
 
 #ifdef HAVE_PTHREAD_BARRIERS
-  // Guaranteed to not be in use when (status_ != PAUSE).
   pthread_barrier_t pause_barrier_;
+  pthread_rwlock_t pause_rwlock_;  // Guards pause_barrier_
 #endif
 
   DISALLOW_COPY_AND_ASSIGN(WorkerStatus);
@@ -235,13 +237,11 @@ class WorkerThread {
   virtual bool Work();
 
   // Starts per-WorkerThread timer.
-  void StartThreadTimer() {gettimeofday(&start_time_, NULL);}
+  void StartThreadTimer() {start_time_ = sat_get_time_us();}
   // Reads current timer value and returns run duration without recording it.
   int64 ReadThreadTimer() {
-    struct timeval end_time_;
-    gettimeofday(&end_time_, NULL);
-    return (end_time_.tv_sec - start_time_.tv_sec)*1000000ULL +
-      (end_time_.tv_usec - start_time_.tv_usec);
+    int64 end_time_ = sat_get_time_us();
+    return end_time_ - start_time_;
   }
   // Stops per-WorkerThread timer and records thread run duration.
   // Start/Stop ThreadTimer repetitively has cumulative effect, ie the timer
@@ -320,6 +320,7 @@ class WorkerThread {
   // Compare a region of memory with a known data patter, and report errors.
   virtual int CheckRegion(void *addr,
                           class Pattern *pat,
+                          uint32 lastcpu,
                           int64 length,
                           int offset,
                           int64 patternoffset);
@@ -383,7 +384,7 @@ class WorkerThread {
   bool tag_mode_;                   // Tag cachelines with vaddr.
 
   // Thread timing variables.
-  struct timeval start_time_;        // Worker thread start time.
+  int64 start_time_;                 // Worker thread start time.
   volatile int64 runduration_usec_;  // Worker run duration in u-seconds.
 
   // Function passed to pthread_create.
